package com.statusbar;

import android.app.Activity;
import android.content.res.Resources;
import android.graphics.Color;
import android.os.Build;
import android.support.annotation.RequiresApi;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaInterface;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CordovaWebView;
import org.apache.cordova.LOG;
import org.json.JSONArray;
import org.json.JSONException;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by linyingqi on 2018/4/12.
 */

public class GizStatusBarManager extends CordovaPlugin {
    private Map<String, CallbackContext> callbackContextMap = new HashMap<String, CallbackContext>();
    private JSONArray requestArgs;


    @Override
    public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {
        this.requestArgs = args;
        if (action.equals("setDefaultStyle")) {
            setStatusBarStyle("blacktranslucent");
            callbackContext.success("");
            return true;
        } else if (action.equals("setLightContentStyle")) {
            setStatusBarStyle("lightcontent");
            callbackContext.success("");
        } else if ("show".equals(action)) {
            this.cordova.getActivity().runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    Window window = cordova.getActivity().getWindow();
                    // SYSTEM_UI_FLAG_FULLSCREEN is available since JellyBean, but we
                    // use KitKat here to be aligned with "Fullscreen"  preference
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                        int uiOptions = window.getDecorView().getSystemUiVisibility();
                        uiOptions &= ~View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN;
                        uiOptions &= ~View.SYSTEM_UI_FLAG_FULLSCREEN;

                        window.getDecorView().setSystemUiVisibility(uiOptions);
                    }

                    // CB-11197 We still need to update LayoutParams to force status bar
                    // to be hidden when entering e.g. text fields
                    window.clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
                }
            });
            return true;
        } else if ("hide".equals(action)) {
            this.cordova.getActivity().runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    Window window = cordova.getActivity().getWindow();
                    // SYSTEM_UI_FLAG_FULLSCREEN is available since JellyBean, but we
                    // use KitKat here to be aligned with "Fullscreen"  preference
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                        int uiOptions = window.getDecorView().getSystemUiVisibility()
                                | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                                | View.SYSTEM_UI_FLAG_FULLSCREEN;

                        window.getDecorView().setSystemUiVisibility(uiOptions);
                    }

                    // CB-11197 We still need to update LayoutParams to force status bar
                    // to be hidden when entering e.g. text fields
                    window.addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
                }
            });
            return true;
        } else if ("statusBarHeight".equals(action)) {
            getStatusBarHeight(callbackContext);
            return true;
        }
        return false;
    }

    @Override
    public void initialize(final CordovaInterface cordova, CordovaWebView webView) {
        super.initialize(cordova, webView);
        this.cordova.getActivity().runOnUiThread(new Runnable() {
            @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
            @Override
            public void run() {
                setStatusBarStyle("blacktranslucent");
//
//                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
//                    Window window = cordova.getActivity().getWindow();
//                    window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS);
//                    window.setStatusBarColor(Color.TRANSPARENT);
//                    window.getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN | View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR);
//                    setXiaoMiStatusBarDarkMode(true);
//                    FlymeSetStatusBarLightMode(true);
//                }
            }
        });
    }

    //  private void getStatusBarHeight(final CallbackContext callbackContext) {
    //     int height = 0;
    //     int status = cordova.getActivity().getWindow().getDecorView().getSystemUiVisibility();
    //     if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP&&(status == View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN||status==(View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN | View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR))) {
    //         Resources resources = cordova.getActivity().getResources();
    //         int resourceId = resources.getIdentifier("status_bar_height", "dimen", "android");
    //         height = resources.getDimensionPixelSize(resourceId);
    //         Log.v("dbw", "Status height:" + height);
    //     }
    //     callbackContext.success(height);
    // }

     private void getStatusBarHeight(final CallbackContext callbackContext) {
        int height = 0;
        int status = cordova.getActivity().getWindow().getDecorView().getSystemUiVisibility();
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP&&(status == View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN||status==(View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN | View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR))) {
            Resources resources = cordova.getActivity().getResources();
            int resourceId = resources.getIdentifier("status_bar_height", "dimen", "android");
            height = resources.getDimensionPixelSize(resourceId);
            Log.v("dbw", "Status height:" + height);
        }
        // else if(Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP)
        // {
        //     Resources resources = cordova.getActivity().getResources();
        //     int resourceId = resources.getIdentifier("status_bar_height", "dimen", "android");
        //     height = resources.getDimensionPixelSize(resourceId);
        //     Log.v("dbw", "Status height:" + height);
        // }
        callbackContext.success(height);
    }

    public void setXiaoMiStatusBarDarkMode(boolean darkmode) {
//        if (sIsMiuiV6) {
        Class<? extends Window> clazz = cordova.getActivity().getWindow().getClass();
        try {
            int darkModeFlag = 0;
            Class<?> layoutParams = Class.forName("android.view.MiuiWindowManager$LayoutParams");
            Field field = layoutParams.getField("EXTRA_FLAG_STATUS_BAR_DARK_MODE");
            darkModeFlag = field.getInt(layoutParams);
            Method extraFlagField = clazz.getMethod("setExtraFlags", int.class, int.class);
            extraFlagField.invoke(cordova.getActivity().getWindow(), darkmode ? darkModeFlag : 0, darkModeFlag);
        } catch (Exception e) {
            e.printStackTrace();
        }
//        }
    }

    /**
     * Starts an intent to scan and decode a barcode.
     */


    private void setStatusBarStyle(final String style) {
        cordova.getActivity().runOnUiThread(new Runnable() {
            @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
            @Override
            public void run() {
                Log.e("rom",Build.BRAND+" "+Build.MANUFACTURER);
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP&&!Build.BRAND.contains("vivo")&&!Build.MANUFACTURER.contains("vivo")) {
                    if (style != null && !style.isEmpty()) {
                        Log.e("改变状态栏字体颜色", style);
                        cordova.getActivity().getWindow().addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);//透明状态栏
                        // 状态栏字体设置为深色，SYSTEM_UI_FLAG_LIGHT_STATUS_BAR 为SDK23增加
                        // 部分机型的statusbar会有半透明的黑色背景
                        cordova.getActivity().getWindow().addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS);
                        cordova.getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
                        cordova.getActivity().getWindow().setStatusBarColor(Color.TRANSPARENT);// SDK21
                        if ("blacktranslucent".equals(style.toLowerCase())) {
                            cordova.getActivity().getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN | View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR);
                            setXiaoMiStatusBarDarkMode(true);
                            FlymeSetStatusBarLightMode(true);
                            return;
                        }
                        if ("lightcontent".equals(style.toLowerCase())) {
                            cordova.getActivity().getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN);
                            setXiaoMiStatusBarDarkMode(false);
                            FlymeSetStatusBarLightMode(false);
                            return;
                        }

                    }
                }
            }
        });

    }

    /**
     * 设置魅族手机状态栏图标颜色风格
     * 可以用来判断是否为Flyme用户
     *
     * @param dark 是否把状态栏字体及图标颜色设置为深色
     * @return boolean 成功执行返回true
     */
    public void FlymeSetStatusBarLightMode(boolean dark) {
        if (cordova.getActivity().getWindow() != null) {
            try {
                WindowManager.LayoutParams lp = cordova.getActivity().getWindow().getAttributes();
                Field darkFlag = WindowManager.LayoutParams.class
                        .getDeclaredField("MEIZU_FLAG_DARK_STATUS_BAR_ICON");
                Field meizuFlags = WindowManager.LayoutParams.class
                        .getDeclaredField("meizuFlags");
                darkFlag.setAccessible(true);
                meizuFlags.setAccessible(true);
                int bit = darkFlag.getInt(null);
                int value = meizuFlags.getInt(lp);
                if (dark) {
                    value |= bit;
                } else {
                    value &= ~bit;
                }
                meizuFlags.setInt(lp, value);
                cordova.getActivity().getWindow().setAttributes(lp);
            } catch (Exception e) {

            }
        }
    }


}
