//
//  GizStatusBarManager.m
//  myApp
//
//  Created by Minus🍀 on 2018/4/11.
//

#import "GizStatusBarManager.h"

#import <objc/runtime.h>
#import <Cordova/CDVViewController.h>

static const void *kGizHideStatusBar = &kGizHideStatusBar;
static const void *kGizStatusBarStyle = &kGizStatusBarStyle;

@interface CDVViewController (StatusBar)

@property (nonatomic, strong) NSNumber *giz_statusBarStyle;
@property (nonatomic, retain) NSNumber *giz_hideStatusBar;


@end

@implementation CDVViewController (StatusBar)

@dynamic giz_statusBarStyle;
@dynamic giz_hideStatusBar;

- (NSNumber *)giz_hideStatusBar {
    return objc_getAssociatedObject(self, kGizHideStatusBar);
}

- (void)setGiz_hideStatusBar:(NSNumber *)newHideStatusBar {
    objc_setAssociatedObject(self, kGizHideStatusBar, newHideStatusBar, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (NSNumber *)giz_statusBarStyle {
    return objc_getAssociatedObject(self, kGizStatusBarStyle);
}

- (void)setGiz_statusBarStyle:(NSNumber *)newStatusBarStyle {
    objc_setAssociatedObject(self, kGizStatusBarStyle, newStatusBarStyle, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (BOOL)prefersStatusBarHidden {
    return self.giz_hideStatusBar ? self.giz_hideStatusBar.boolValue : NO;
}

- (UIStatusBarStyle)preferredStatusBarStyle {
    return (UIStatusBarStyle)self.giz_statusBarStyle.integerValue;
}

@end


@implementation GizStatusBarManager

- (void)setStatusBarStyle:(UIStatusBarStyle)style {
    if ([self.viewController isKindOfClass:[CDVViewController class]]) {
        CDVViewController *viewController = (CDVViewController *)self.viewController;
        
        viewController.giz_statusBarStyle = @(style);
        
        [viewController setNeedsStatusBarAppearanceUpdate];
    }
}

- (void)showStatusBar {
    if ([self.viewController isKindOfClass:[CDVViewController class]]) {
        CDVViewController *vc = (CDVViewController *)self.viewController;
        vc.giz_hideStatusBar = @NO;
        [self refreshStatusBarAppearance];
    }
}

- (void)show:(CDVInvokedUrlCommand *)command {
    
    UIApplication *app = [UIApplication sharedApplication];
    
    if (app.isStatusBarHidden) {
        
        [self showStatusBar];
    }
}

- (void)hideStatusBar {
    if ([self.viewController isKindOfClass:[CDVViewController class]]) {
        CDVViewController *vc = (CDVViewController *)self.viewController;
        vc.giz_hideStatusBar = @YES;
        [self refreshStatusBarAppearance];
    }
}

- (void)hide:(CDVInvokedUrlCommand *)command {
    
    UIApplication *app = [UIApplication sharedApplication];
    
    if (!app.isStatusBarHidden) {
        
        [self hideStatusBar];
    }
}

- (void)refreshStatusBarAppearance {
    
    [self.viewController setNeedsStatusBarAppearanceUpdate];
}

- (void)setDefaultStyle:(CDVInvokedUrlCommand *)command {
    [self setStatusBarStyle:UIStatusBarStyleDefault];
}

- (void)setLightContentStyle:(CDVInvokedUrlCommand *)command {
    [self setStatusBarStyle:UIStatusBarStyleLightContent];
}

@end
